Imports ADOX
Imports System.Data.OleDb
Imports System.Xml

Module MDB

    Dim Priv_MDBFileName As String = System.AppDomain.CurrentDomain.BaseDirectory & "winfail2ban.mdb"
    Dim Priv_MDBStayConnected As Boolean = True

    Dim MDBConn As OleDb.OleDbConnection
    Dim MDBLastLog As String = ""


#Region "MDB Function"

    'Create Database 
    Private Sub CreateDB(ByVal dbPath As String)
        Try
            Dim dbCatalog As New ADOX.Catalog
            dbCatalog.Create("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" & dbPath)
            TextFileLogging("Database File created succesfully !")
            Dim MDBConn1 As New OleDb.OleDbConnection(ConnectionString())
            CreateTableSchema(MDBConn1)
            TextFileLogging("Database Schema created succesfully !")
        Catch sqlexp As Exception
            TextFileLogging("Create Database fail" & vbCrLf & sqlexp.Message)
            MDBLastLog = sqlexp.ToString
        End Try
    End Sub

    'Create Table
    Private Sub CreateTableSchema(ByVal MDBConn As OleDb.OleDbConnection)
        Dim SQLString As String
        Dim cmd As New OleDb.OleDbCommand
        If MDBConn.State <> ConnectionState.Open Then
            MDBConn.Open()
        End If
        Try
            'Generate SQL String for Table Creation
            SQLString = "CREATE TABLE Attacks ( " & _
                        "[ID] int IDENTITY  NOT NULL , " & _
                        "[IPAddress] varchar(15) NOT NULL , " & _
                        "[Plugin] int NOT NULL , " & _
                        "[AttackDate] datetime NOT NULL " & _
                        ")"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "CREATE TABLE BlackList ( " & _
            "[ID] int IDENTITY  NOT NULL , " & _
            "[IPAddress] varchar(15) NOT NULL , " & _
            "[BannedDate] datetime NOT NULL , " & _
            "[Note] varchar(50) NULL  " & _
            ")"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "CREATE TABLE WhiteList ( " & _
            "[ID] int IDENTITY  NOT NULL , " & _
            "[IPAddress] varchar(15) NOT NULL , " & _
            "[Note] varchar(50) NULL , " & _
            "[InsertDate] datetime NOT NULL " & _
            ")"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "CREATE TABLE Plugins ( " & _
            "[ID] int IDENTITY  NOT NULL , " & _
            "[PluginName] varchar(15) NOT NULL , " & _
            "[PluginID] int NOT NULL  " & _
            ")"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "INSERT INTO [Plugins] ([PluginName],[PluginID]) " & _
                        "VALUES ('RDP', 1)"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "INSERT INTO [Plugins] ([PluginName],[PluginID]) " & _
            "VALUES ('MSSQL', 2)"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

            SQLString = "INSERT INTO [Plugins] ([PluginName],[PluginID]) " & _
            "VALUES ('FTP', 3)"
            cmd = New OleDbCommand(SQLString, MDBConn)
            cmd.ExecuteNonQuery()

        Catch sqlexp As OleDb.OleDbException
            TextFileLogging(sqlexp.ToString)
        End Try
    End Sub

    ' Check DB Existance 
    Private Function CheckDBExistance(ByVal MDBConn As OleDb.OleDbConnection) As Boolean
        Try
            MDBConn.Open()
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function

    Private Function ConnectionString() As String
        ' Connection string could use Initial Catalog to specify Database
        ' Example 
        '  "Data Source=%sqlservername%;Initial Catalog=%database%;Integrated Security=SSPI"
        ' See www.able-consulting.com for reference.
        If (Priv_MDBFileName = "") Then
            ConnectionString = "Missing File Name"
        Else
            ConnectionString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=%filename%;"
        End If
        ConnectionString = Replace(ConnectionString, "%filename%", Priv_MDBFileName)
    End Function

#End Region

#Region "Manage SQL Connection"

    ' Check Connection to SQL Server 
    Function MDBTestConnection() As Boolean
        Dim connect_string As String
        Try
            If Priv_MDBFileName = "" Then
                TextFileLogging("Missing parameters")
                Return False
            End If

            connect_string = ConnectionString()
            Dim MDBConn As New OleDb.OleDbConnection(connect_string)
            'MDBConn.Open()
            If CheckDBExistance(MDBConn) = False Then
                If MsgBox("Database doesn't exist, do you want to create it now ?", MsgBoxStyle.YesNo, "WinFail2Ban") = MsgBoxResult.Yes Then
                    CreateDB(Priv_MDBFileName)
                    MDBConn.Close()
                Else
                    MDBConn.Close()
                    TextFileLogging("DB Missing")
                End If
                Return False
            End If
            Return True
        Catch ex As OleDb.OleDbException
            MDBLastLog = ex.ToString
            TextFileLogging("Connection failed" & vbCrLf & MDBLastLog)
            Return False
        End Try
    End Function

    ' Start SQL Connection
    Sub StartConn()
        Try
            MDBConn.Open()
            'If (Not Priv_StayConnected) Then MSSQLConn.Close()
        Catch ex As Exception
            MDBLastLog = ex.ToString
        End Try
    End Sub

    ' Stop SQL Connection
    Sub StopConn()
        Try
            If (MDBConn.State = ConnectionState.Open) Then MDBConn.Close()
        Catch ex As Exception
            MDBLastLog = ex.ToString
        End Try
    End Sub

#End Region

#Region "INSERT Procedures"

    Public Function MDBInsertIP(ByVal IPAddress As String, ByVal Plugin As Integer, ByVal AttackDate As DateTime) As Boolean
        Dim sqlstring As String = ""
        Dim cmd As OleDb.OleDbCommand
        Try
            '****************************************************
            '*       TO DO: FINISH VALIDATE PARAMETERS          *
            '****************************************************
            If IPAddress = "" Or Len(IPAddress) > 15 Then
                Exit Function
            End If

            ' GENERATE SQL STRING
            sqlstring = "INSERT INTO [Attacks]([IPAddress], [Plugin], [AttackDate]) " & _
                        "VALUES('" & IPAddress & "', " & Plugin & ", '" & AttackDate & "')"

            If MDBConn Is Nothing Then
                MDBConn = New OleDb.OleDbConnection(ConnectionString())
                MDBConn.Open()
            ElseIf MDBConn.State <> ConnectionState.Open Then
                StartConn()
            End If
            cmd = New OleDbCommand(sqlstring, MDBConn)
            ' EXECUTE QUERY
            Try
                cmd.ExecuteNonQuery()
            Catch SQLExc As OleDb.OleDbException
                If SQLExc.ErrorCode = -2147217865 Then
                    CreateTableSchema(MDBConn)
                    If Not sqlstring = "" Then
                        Try
                            cmd.CommandText = sqlstring
                            cmd.ExecuteNonQuery()
                        Catch ex As Exception

                        End Try
                    End If
                Else

                End If

            End Try
            cmd.Dispose()

            If (Not Priv_MDBStayConnected) Then StopConn()
            Return True

        Catch ex As Exception
            '****************************************************
            '* TO DO .... CHECK IF CONNECTION HAS BEEN RESETTED *
            '****************************************************
            If ex.Message = "Could not find file '" & Priv_MDBFileName & "'." Then
                CreateDB(Priv_MDBFileName)
            Else
                TextFileLogging(ex.ToString)
            End If
            Return False
        End Try
    End Function

    Public Function MDBBlackListIP(ByVal IPAddress As String, ByVal AttackDate As DateTime, ByVal Note As String) As Boolean
        Dim sqlstring As String = ""
        Dim cmd As OleDb.OleDbCommand
        Try
            '****************************************************
            '*       TO DO: FINISH VALIDATE PARAMETERS          *
            '****************************************************
            If IPAddress = "" Or Len(IPAddress) > 15 Then
                Exit Function
            End If

            ' GENERATE SQL STRING
            sqlstring = "INSERT INTO [BlackList]([IPAddress], [BannedDate], [Note]) " & _
                        "VALUES('" & IPAddress & "', '" & AttackDate & "', '" & Note & "')"
            If MDBConn Is Nothing Then
                MDBConn = New OleDb.OleDbConnection(ConnectionString())
                MDBConn.Open()
            ElseIf MDBConn.State <> ConnectionState.Open Then
                StartConn()
            End If
            cmd = New OleDbCommand(sqlstring, MDBConn)
            ' EXECUTE QUERY
            Try
                cmd.ExecuteNonQuery()
            Catch SQLExc As OleDb.OleDbException
                If SQLExc.ErrorCode = -2147217865 Then
                    CreateTableSchema(MDBConn)
                    If Not sqlstring = "" Then
                        Try
                            cmd.CommandText = sqlstring
                            cmd.ExecuteNonQuery()
                        Catch ex As Exception

                        End Try
                    End If
                Else
                    TextFileLogging(SQLExc.Message)
                End If

            End Try
            cmd.Dispose()

            If (Not Priv_MDBStayConnected) Then StopConn()
            Return True

        Catch ex As Exception
            '****************************************************
            '* TO DO .... CHECK IF CONNECTION HAS BEEN RESETTED *
            '****************************************************
            If ex.Message = "Could not find file '" & Priv_MDBFileName & "'." Then
                CreateDB(Priv_MDBFileName)
            Else
                TextFileLogging(ex.ToString)
            End If
            Return False
        End Try
    End Function

#End Region

#Region "CHECK IP Procedure"

    Public Function MDBCheckIP(ByVal IPAddress As String, ByVal TimeThreshold As Integer) As Long
        Dim sqlstring As String = ""
        Dim result As Long
        Dim cmd As OleDb.OleDbCommand
        Try
            '****************************************************
            '*       TO DO: FINISH VALIDATE PARAMETERS          *
            '****************************************************
            If IPAddress = "" Or Len(IPAddress) > 15 Then
                Return 0
            End If

            ' GENERATE SQL STRING
            sqlstring = "SELECT COUNT(IPAddress) FROM [Attacks] " & _
            " WHERE DATEDIFF(""n"", AttackDate, Now()) < " & TimeThreshold & _
            " AND (IPAddress NOT IN (SELECT IPAddress FROM WhiteList))"

            '****************************************************
            '*       TO DO: VERIFY WITH NOLOCK CLAUSE           *
            '****************************************************

            If MDBConn Is Nothing Then
                MDBConn = New OleDb.OleDbConnection(ConnectionString())
                MDBConn.Open()
            ElseIf MDBConn.State <> ConnectionState.Open Then
                StartConn()
            End If
            cmd = New OleDbCommand(sqlstring, MDBConn)
            ' EXECUTE QUERY
            Try
                result = cmd.ExecuteScalar

            Catch SQLExc As OleDb.OleDbException
                If SQLExc.ErrorCode = -2147217865 Then
                    CreateTableSchema(MDBConn)
                    If Not sqlstring = "" Then
                        Try
                            cmd.CommandText = sqlstring
                            result = cmd.ExecuteScalar
                        Catch ex As Exception

                        End Try
                    End If
                Else

                End If

            End Try
            cmd.Dispose()

            If (Not Priv_MDBStayConnected) Then StopConn()
            Return result

        Catch ex As Exception
            '****************************************************
            '* TO DO .... CHECK IF CONNECTION HAS BEEN RESETTED *
            '****************************************************
            If ex.Message = "Could not find file '" & Priv_MDBFileName & "'." Then
                CreateDB(Priv_MDBFileName)
            End If
            MDBLastLog = ex.ToString
            Return 0
        End Try
    End Function


#End Region


#Region "Utilities"

    Public Function TextFileLogging(ByVal MessageString As String)
        ' Write to text log file
        Dim filesys, filetxt
        Const ForReading = 1, ForWriting = 2, ForAppending = 8
        Try
            filesys = CreateObject("Scripting.FileSystemObject")
            filetxt = filesys.OpenTextFile(System.AppDomain.CurrentDomain.BaseDirectory() & "\log.txt", ForAppending, True)
            filetxt.writeline(Now() & vbCrLf & "WinFail2Ban: " & MessageString & vbCrLf & "-------------------------")
            filetxt.Close()
        Catch exc As Exception
            'Do nothing
        End Try
    End Function

#End Region

End Module
