Public Class Firewall

    '************************************************************************************
    '*   BLOCKING TECHNIQUES
    '*
    '*   IPTABLES:    iptables -A FORWARD -s <IP> -j DROP
    '*   BAD ROUTE:   "ROUTE ADD " & IP & " MASK 255.255.255.255 " & fakeGateWay
    '*
    '************************************************************************************

    Private _IPTable, _BadRoute, _VerboseLogging As Boolean
    Private _BadGateway

    Public Sub New()
        _VerboseLogging = False
        _IPTable = False
        _BadRoute = False
        _BadGateway = ObtainIPAddress()
    End Sub

    Public Property VerboseLogging() As Boolean
        Get
            Return _VerboseLogging
        End Get
        Set(ByVal Value As Boolean)
            _VerboseLogging = Value
        End Set
    End Property

    Public Property IPTable() As Boolean
        Get
            Return _IPTable
        End Get
        Set(ByVal Value As Boolean)
            _IPTable = Value
        End Set
    End Property

    Public Property BadRoute() As Boolean
        Get
            Return _BadRoute
        End Get
        Set(ByVal Value As Boolean)
            _BadRoute = Value
        End Set
    End Property

    Public Property BadGateway() As String
        Get
            Return _BadGateway
        End Get
        Set(ByVal Value As String)
            _BadGateway = Value
        End Set
    End Property

    Public Sub BanIP(ByVal IPAddress As String)
        Try
            If _VerboseLogging = True Then
                TextFileLogging("BanIP " & IPAddress)
            End If
            If _IPTable = True Then
                Ban_IPTable(IPAddress)
            End If
            If _BadRoute = True Then
                Ban_BadRoute(IPAddress)
            End If
        Catch ex As Exception

        End Try
    End Sub


#Region "Private Routines"

    Private Sub Ban_IPTable(ByVal IPAddress As String)
        'iptables -A FORWARD -s <IP> -j DROP
        Try
            If _VerboseLogging = True Then
                TextFileLogging("Ban_IPTable " & IPAddress)
            End If
        Catch ex As Exception

        End Try
    End Sub

    Private Sub Ban_BadRoute(ByVal IPAddress As String)
        'BAD ROUTE:   "ROUTE ADD " & IP & " MASK 255.255.255.255 " & fakeGateWay
        Try
            If _VerboseLogging = True Then
                TextFileLogging("Ban_BadRoute " & IPAddress)
            End If
            Dim name As String = System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
            EventLog.WriteEntry(name, "Banned IP " & IPAddress & " using BAD ROUTE to " & _BadGateway)
            Dim pr As New Process
            Dim StdOutput As String = ""
            pr.StartInfo.FileName = "cmd.exe"
            pr.StartInfo.Arguments = "/C ROUTE ADD " & IPAddress & " MASK 255.255.255.255 " & _BadGateway
            pr.StartInfo.UseShellExecute = False
            pr.StartInfo.RedirectStandardOutput = True
            pr.Start()
            StdOutput = pr.StandardOutput.ReadToEnd
            pr.WaitForExit(30000)  'Maximum time is 30 seconds
            If _VerboseLogging = True Then
                TextFileLogging("Ban_BadRoute Output: " & StdOutput)
            End If
        Catch ex As Exception
            TextFileLogging(ex.ToString)
        End Try
    End Sub

    Private Function ObtainIPAddress() As String
        'Get IP address of server so we can use it later to give the offending IP a bad route
        Try
            If _VerboseLogging = True Then
                TextFileLogging("ObtainIPAddress")
            End If
            Dim IPConfigSet = GetObject("winmgmts:\\.\root\cimv2").ExecQuery("SELECT * FROM Win32_NetworkAdapterConfiguration WHERE IPEnabled=TRUE")
            Dim ipconfig
            For Each ipconfig In IPConfigSet
                If _VerboseLogging = True Then
                    TextFileLogging("ObtainIPAddress IP: " & ipconfig.IPAddress(0))
                End If
                If Not IsDBNull(ipconfig.DefaultIPGateway) Then Return ipconfig.IPAddress(0)
            Next
            IPConfigSet = Nothing
            Return "127.0.0.1"
        Catch ex As Exception
            Return "127.0.0.1"
        End Try
    End Function


#End Region

End Class
