; Instructions
; ------------
; This script is used by the Nullsoft Scriptable Install System (NSIS) which is an open source project at: http://nsis.sourceforge.net
; It is released under an open source license and is completely free for any use.

; To compile the script download and install the NSIS: http://nsis.sourceforge.net
; Download the Service Control Manager plugin: http://nsis.sourceforge.net/archive/viewpage.php?pageid=385
; This plugin allows services to be controlled on NT/2K/XP. Copy the nsSCM.dll file into C:\Program Files\NSIS\Plugins


; To edit the script download the HM NIS EDIT at: http://hmne.sourceforge.net
; It is a Free NSIS Editor/IDE for NSIS.

; HM NIS Edit Wizard helper defines
!define PRODUCT_NAME "WinFail2Ban"
!define PRODUCT_VERSION "0.2"
!define PRODUCT_PUBLISHER "WinFail2Ban Project"
!define PRODUCT_WEB_SITE "http://winfail2ban.sourceforge.net"
!define PRODUCT_DIR_REGKEY "SOFTWARE\WinFail2Ban"
!define PRODUCT_UNINST_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PRODUCT_NAME}"
!define PRODUCT_UNINST_ROOT_KEY "HKLM"

!define DOT_MAJOR "2"
!define DOT_MINOR "0"

SetCompressor lzma

; MUI 1.67 compatible ------
!include "MUI.nsh"

; Needed for WMI Scripting Library Installation
!include Library.nsh

; MUI Settings
!define MUI_ABORTWARNING

!define MUI_ICON "installer.ico"
!define MUI_UNICON "uninstaller.ico"
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "header.bmp"
!define MUI_WELCOMEFINISHPAGE_BITMAP "wizard.bmp"

; Welcome page
!insertmacro MUI_PAGE_WELCOME
; License page
!insertmacro MUI_PAGE_LICENSE "License.rtf"
; Components page
!insertmacro MUI_PAGE_COMPONENTS
; Directory page
!insertmacro MUI_PAGE_DIRECTORY
; Instfiles page
!insertmacro MUI_PAGE_INSTFILES
; Finish page
#!define MUI_FINISHPAGE_RUN "$INSTDIR\Interface.exe"
!insertmacro MUI_PAGE_FINISH

; Uninstaller pages
!insertmacro MUI_UNPAGE_INSTFILES

; Language files
!insertmacro MUI_LANGUAGE "English"

; Reserve files
!insertmacro MUI_RESERVEFILE_INSTALLOPTIONS


; MUI end ------

Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
OutFile "WinFail2Ban_Setup.exe"

InstType "Full Installation"
InstType "Service only"

BrandingText "${PRODUCT_NAME} ${PRODUCT_VERSION} Setup"
InstallDir "$PROGRAMFILES\WinFail2Ban"
InstallDirRegKey HKLM "${PRODUCT_DIR_REGKEY}" ""

ShowInstDetails show
ShowUnInstDetails show

Section "Windows Service (Required)" WinFail2BanSEC
  SectionIn 1 2 3 RO
  
  SetOutPath "$INSTDIR"
  SetOverwrite ifnewer
  File "License.rtf"
  File "..\Service\bin\Release\WinFail2Ban.exe"
  File "..\Service\bin\Release\winfail2ban.exe.config"
  File "..\Service\bin\Release\DataLayer.dll"
  File "..\Service\bin\Release\Interop.ADOX.dll"
  File "..\Service\bin\Release\Interop.MSUtil.dll"
  File "..\Service\bin\Release\LogParser.dll"
  File "..\Resources\WinFail2Ban.chm"
  
  RegDLL "$INSTDIR\LogParser.dll"

;  CreateDirectory "$SMPROGRAMS\WinFail2Ban"
;  CreateShortCut "$SMPROGRAMS\WinFail2Ban\WinFail2Ban.lnk" "$INSTDIR\WinFail2BanGUI.exe"

  ; Delete previous settings
  DeleteRegKey HKLM "${PRODUCT_DIR_REGKEY}"

SectionEnd

Section "User Interface" GUISEC
  SectionIn 1 ;RO
  File "..\WinFail2BanGUI\bin\Release\WinFail2BanGUI.exe"

SectionEnd

Section -AdditionalIcons
  WriteIniStr "$INSTDIR\${PRODUCT_NAME}_HomePage.url" "InternetShortcut" "URL" "${PRODUCT_WEB_SITE}"
  CreateDirectory "$SMPROGRAMS\WinFail2Ban"
  CreateShortCut "$SMPROGRAMS\WinFail2Ban\HomePage.lnk" "$INSTDIR\${PRODUCT_NAME}_HomePage.url"
  CreateShortCut "$SMPROGRAMS\WinFail2Ban\Log.lnk" "$INSTDIR\Log.txt"
  CreateShortCut "$SMPROGRAMS\WinFail2Ban\WinFail2Ban.lnk" "$INSTDIR\WinFail2BanGUI.exe"
  CreateShortCut "$SMPROGRAMS\WinFail2Ban\Manual.lnk" "$INSTDIR\WinFail2Ban.chm"
  CreateShortCut "$SMPROGRAMS\WinFail2Ban\Uninstall.lnk" "$INSTDIR\Uninstall.exe"
SectionEnd

Section -Post
  WriteUninstaller "$INSTDIR\Uninstall.exe"
  WriteRegStr HKLM "${PRODUCT_DIR_REGKEY}" "" "$INSTDIR\WinFail2Ban.exe"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayName" "$(^Name)"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString" "$INSTDIR\Uninstall.exe"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayIcon" "$INSTDIR\WinFail2Ban.exe"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayVersion" "${PRODUCT_VERSION}"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "URLInfoAbout" "${PRODUCT_WEB_SITE}"
  WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "Publisher" "${PRODUCT_PUBLISHER}"

  nsSCM::Install /NOUNLOAD "WinFail2Ban" "WinFail2Ban" 16 2 "$INSTDIR\WinFail2Ban.exe" "" "" "" ""
  Pop $0 ; return error/success
;  MessageBox MB_ICONINFORMATION $0
  WriteRegStr HKLM "SYSTEM\CurrentControlSet\Services\WinFail2Ban" "Description" "Scans log files like FTP Logs or Event Viewer and bans IP that makes too many password failures"
  Sleep 100
SectionEnd

Function .onInit

Call IsDotNetInstalled

FunctionEnd

Function .onInstSuccess
    MessageBox MB_YESNO "Thank you for using WinFail2Ban, Do you want to start the WinFail2Ban Service?" IDNO NoStart

    ;Start the Service......	
    nsSCM::Start /NOUNLOAD "WinFail2Ban"
    Pop $0 ; return error/success

    NoStart:
FunctionEnd


Function un.onUninstSuccess
  HideWindow
  MessageBox MB_ICONINFORMATION|MB_OK "$(^Name) was successfully removed from your computer."

  UnRegDLL "$INSTDIR\LogParser.dll"
  
  Delete "$INSTDIR\log.txt"
  Delete "$INSTDIR\LogParser.dll"
  Delete "$INSTDIR\${PRODUCT_NAME}_HomePage.url"
  Delete "$INSTDIR\Uninstall.exe"
  Delete "$INSTDIR\winfail2ban.mdb"
  Delete "$INSTDIR\WinFail2BanGUI.exe"
  Delete "$INSTDIR\DataLayer.dll"
  Delete "$INSTDIR\WinFail2Ban.chm"
  
  RmDir "$INSTDIR"
FunctionEnd

Function un.onInit
  MessageBox MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2 "Are you sure you want to completely remove $(^Name) and all of its components?" IDYES +2
  Abort
FunctionEnd

Section Uninstall

  ; kill winfail2bangui.exe process (Graphical User Interface)
  ;Processes::KillProcess "winfail2banGUI"
  ;Pop $0 ; return error/success

  ; <name of service:startstop name>
  nsSCM::Stop /NOUNLOAD "WinFail2Ban"
  Pop $0 ; return error/success

  ; <name of service:startstop name>
  nsSCM::Remove /NOUNLOAD "WinFail2Ban"
  Pop $0 ; return error/success

  Delete "$INSTDIR\${PRODUCT_NAME}_HomePage.url"
  Delete "$INSTDIR\Uninstall.exe"

; TO DO: REMOVE ALL DLL

  RMDir /r "$SMPROGRAMS\WinFail2Ban"
  RMDir /r "$INSTDIR"
  
  DeleteRegKey ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}"
  DeleteRegKey HKLM "${PRODUCT_DIR_REGKEY}"

  SetAutoClose true
SectionEnd



;***********************************************************************************

; Usage
; Define in your script two constants:
;   DOT_MAJOR "(Major framework version)"
;   DOT_MINOR "{Minor frameword version)"
; 
; Call IsDotNetInstalled
; This function will abort the installation if the required version 
; or higher version of the .NETFramework is not installed.  Place it in
; either your .onInit function or your first install section before 
; other code.

Function IsDotNetInstalled

  StrCpy $0 "0"
  StrCpy $1 "SOFTWARE\Microsoft\.NETFramework" ;registry entry to look in.
  StrCpy $2 0

  StartEnum:
    ;Enumerate the versions installed.
    EnumRegKey $3 HKLM "$1\policy" $2
    
    ;If we don't find any versions installed, it's not here.
    StrCmp $3 "" noDotNet notEmpty
    
    ;We found something.
    notEmpty:
      ;Find out if the RegKey starts with 'v'.  
      ;If it doesn't, goto the next key.
      StrCpy $4 $3 1 0
      StrCmp $4 "v" +1 goNext
      StrCpy $4 $3 1 1
      
      ;It starts with 'v'.  Now check to see how the installed major version
      ;relates to our required major version.
      ;If it's equal check the minor version, if it's greater, 
      ;we found a good RegKey.
      IntCmp $4 ${DOT_MAJOR} +1 goNext yesDotNetReg
      ;Check the minor version.  If it's equal or greater to our requested 
      ;version then we're good.
      StrCpy $4 $3 1 3
      IntCmp $4 ${DOT_MINOR} yesDotNetReg goNext yesDotNetReg

    goNext:
      ;Go to the next RegKey.
      IntOp $2 $2 + 1
      goto StartEnum

  yesDotNetReg:
    ;Now that we've found a good RegKey, let's make sure it's actually
    ;installed by getting the install path and checking to see if the 
    ;mscorlib.dll exists.
    EnumRegValue $2 HKLM "$1\policy\$3" 0
    ;$2 should equal whatever comes after the major and minor versions 
    ;(ie, v1.1.4322)
    StrCmp $2 "" noDotNet
    ReadRegStr $4 HKLM $1 "InstallRoot"
    ;Hopefully the install root isn't empty.
    StrCmp $4 "" noDotNet
    ;build the actuall directory path to mscorlib.dll.
    StrCpy $4 "$4$3.$2\mscorlib.dll"
    IfFileExists $4 yesDotNet noDotNet

  noDotNet:
    ;Nope, something went wrong along the way.  Looks like the 
    ;proper .NETFramework isn't installed.  
    MessageBox MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2 "Microsoft .NET Framework version ${DOT_MAJOR}.${DOT_MINOR} or greater is required, do you want download now ?" IDYES +2
    Abort
    ExecShell "open" "http://www.microsoft.com/downloads/details.aspx?FamilyID=262d25e3-f589-4842-8157-034d1e7cf3a3&displaylang=en"
    Abort
  yesDotNet:
    ;Everything checks out.  Go on with the rest of the installation.
    
FunctionEnd

